-- (c) M.P.I 1998/99-2001/02 -- Date: 01.11.20

module BTree where

import Mpi
import List

-- (1) Datatype definition -----------------------------------------------------

data BTree a = Empty | Node(a, (BTree a, BTree a)) deriving Show

inBTree :: Either () (b,(BTree b,BTree b)) -> BTree b
inBTree = either (const Empty) Node

outBTree :: BTree a -> Either () (a,(BTree a,BTree a))
outBTree Empty              = Left ()
outBTree (Node (a,(t1,t2))) = Right(a,(t1,t2))

-- (2) Ana + cata + hylo -------------------------------------------------------

baseBTree f g = id -|- (f >< (g >< g))

cataBTree g = g . (recBTree (cataBTree g)) . outBTree

anaBTree g = inBTree . (recBTree (anaBTree g) ) . g

hyloBTree h g = cataBTree h . anaBTree g

recBTree f = baseBTree id f

-- (3) Map ---------------------------------------------------------------------

instance Functor BTree
         where fmap f = cataBTree ( inBTree . baseBTree f id )

-- (4) Examples ----------------------------------------------------------------

-- (4.1) Quicksort -------------------------------------------------------------

qSort :: Ord a => [a] -> [a]
qSort = hyloBTree inord qsep -- = (cataBTree inord) . (anaBTree sep)

-- where

inord = either (const []) join

join(x,(l,r))=l++[x]++r

qsep []    = Left ()
qsep (h:t) = Right (h,(s,l)) where (s,l) = part (<h) t

part p []                = ([],[])
part p (h:t) | p h       = let (s,l) = part p t in (h:s,l)
             | otherwise = let (s,l) = part p t in (s,h:l)

{-- pointwise version:
qSort [] = []
qSort (h:t) = let (t1,t2) = part (<h) t
	      in  qSort t1 ++ [h] ++ qSort t2
--}

-- (4.2) Traces ----------------------------------------------------------------

traces :: Eq a => BTree a -> [[a]]
traces = cataBTree (either (const [[]]) put)

-- where

put(a,(l,r)) = union (map (a:) l) (map (a:) r) 

-- (4.3) Towers of Hanoi -------------------------------------------------------

hanoi = hyloBTree present strategy

--- where

present = either (const []) join

strategy (0,a,b,c) = Left ()
strategy (n,a,b,c) = Right (a++"->"++c, ((n-1,a,c,b),(n-1,b,a,c)))

--------------------------------------------------------------------------------

