module MonadicPComb where

------------- Tipos -----------

data Error a = Ok a | Erro String deriving Show

data Parse a = P (String -> Error (a,String))

parse :: (Parse a) -> String -> Error a
-- Dados um parser e o input, 
-- retorna (Ok x) se consegue reconhecer x
-- ou entao retorna (Erro s) onde s e a parte do input onde da erro 
parse (P p) input = case (p input) of
                      Ok (x,_) -> Ok x
                      Erro x  -> Erro x

instance Monad Parse where
   return x = P (\input -> Ok (x,input))
   (P p) >>= f  = P (\input -> case (p input) of
                               Ok (a,remaining) -> let (P x) = f a
                                                   in x remaining
                               Erro x -> Erro x
                    )

-----------------  Combinadores ----------------

parseConst :: String -> Parse ()
-- Consome uma constante do input
parseConst c = P (\input -> case (lex input) of 
                            ((x,y):_) -> if (x == c) then Ok ((),y) 
                                                     else Erro input
                            _         -> Erro input)
parseString :: Parse String 
-- o equivalente ao lex
parseString = P (\input -> case (lex input) of
                            ((x,y):_) -> Ok (x,y)
                            _         -> Erro input)

parseStringTo :: Char -> Parse String 
-- consome todo o input até aparecer um determinado caracter
-- da erro se tal nao acontecer
parseStringTo c = P (\input -> case (span (/= c) input ) of
                     (x,[]) -> Erro ("Fim detectado antes de " ++ [c])
                     (a,b) -> Ok (a,b))

parsePar :: (Parse a) -> (Parse b) -> Parse (a,b)
-- o nome e o tipo diz tudo
parsePar p1 p2 = do { x <- p1 ;
                      y <- p2 ;
                      return (x,y) }

ifNot :: (Parse a) -> (Parse a) -> Parse a
-- tenta aplicar o primeiro; em caso de erro da o resultado do segundo
ifNot (P p1) (P p2) = P (\input -> case (p1 input) of
                                   Ok x -> Ok x
                                   _    -> p2 input)

single :: (Parse a) -> Parse [a]
-- so para converter elementos em listas singulares
-- e usado por exemplo quando temos uma lista com separadores
single p = do { x <- p ;
                return [x]
              }

parseSeq :: (Parse a) -> Parse [a]
-- dado um parser de um tipo, aplica-o enquanto puder 
parseSeq p =  ifNot (do { a <- p ;
                          as <- parseSeq p ;
                          return (a:as)
                        }) (return [])

parseSeqSep :: String -> (Parse a) -> Parse [a]
-- igual ao anterior, mas para elementos separados por um separador
parseSeqSep s p =  ifNot (do { a <- p ;
                               parseConst s ;
                               as <- parseSeqSep s p ;
                               return (a:as)
                              }) (ifNot (single p) (return []))
                               
parseParen :: String -> String -> (Parse a) -> Parse a
-- trata do abrir (ch1) e fechar (ch2) de parentesis
parseParen ch1 ch2 p = do { parseConst ch1 ;
                            x <- p ;
                            parseConst ch2 ;
                            return x
                          }

parsePCurvos = parseParen "(" ")"
parsePRectos = parseParen "[" "]"
parseChaveta = parseParen "{" "}"


