\documentclass[10pt]{article}

\usepackage{graphics}
\usepackage{t1enc}
\usepackage{a4wide}
\usepackage{times}
\usepackage{mpi0102t3}

\title{\esimo3 Trabalho Prtico}

\author{
        \mpi{Mtodos de Programao I}
        \\
        \lesi{LESI}/\lmcc{LMCC}
        \\
        Universidade do Minho
}


\date{Ano Lectivo de 2001/2002}

\begin{document}

\maketitle

\section{Introduo}

Um \emph{fractal}  uma estrutura matemtica que se repete
infinitamente, e a que corresponde uma representao visual
``infinitamente detalhada''. Uma imagem possvel  a seguinte: se
usarmos dispositivos de ampliao cada vez mais potentes (lupas,
microscpios), conseguiremos sempre ver cada vez mais detalhes na
figura, que pareciam no existir quando a observvamos com o
dispositivo anterior. Os fractais tm aplicaes importantes que no
sero aqui exploradas, nomeadamente na simulao de fenmenos
naturais (por exemplo climatricos).

Neste trabalho estudar-se- a gerao de fractais por hilomorfismos de
determinados tipos indutivos.

\section{A Biblioteca Grfica \texttt{SOEGraphics}}

Para o desenho dos fractais sugere-se a utilizao da biblioteca
\texttt{SOEGraphics}~\cite{hudak00haskell}\footnote{distribuda como
  parte da \emph{Hugs Graphics Library} em
  \texttt{http://haskell.org/graphics/}}. Para isso basta fazer
\begin{code}
import SOEGraphics
\end{code}
O desenho de objectos grficos no  mais do que um caso especial de
processamento de \emph{input / output} mondico. Apresentam-se aqui
algumas funes bsicas suficientes para o presente trabalho.

Antes de mais, vejamos como criar uma janela grfica utilizando a
seguinte funo:
\begin{verbatim}
type Title = String
type Size = (Int,Int)
openWindow :: Title -> Size -> IO Window
\end{verbatim}

O seguinte programa abre uma janela com nome ``Sierpinski'', de
dimenso 300 $\times$ 300; escreve nela uma mensagem; espera que uma
tecla seja premida; finalmente fecha a janela.
\begin{code}
main =
   runGraphics (
   do w <- openWindow "Sierpinski" (300,300)
      drawInWindow w (text (100,200) "Aqui est uma janela")
      k <- getKey w
      closeWindow w
   )
\end{code}
A funo \verb+drawInWindow :: Window -> Graphic -> IO()+ desenha um
objecto de tipo \verb+Graphic+ numa janela.   naturalmente possvel
desenhar outro tipo de objectos alm de texto; o seguinte ser til
para o desenho dos fractais:
\begin{verbatim}
type Point = (Int,Int)
polygon :: [Point] -> Graphic
\end{verbatim}
Objectos gerados por esta funo correspondem a polgonos descritos
por uma lista de pontos.

Para colorir os fractais sugere-se a utilizao da seguinte funo:
\begin{verbatim}
withColor :: Color -> Graphic -> Graphic
\end{verbatim}

Como exemplo de utilizao, apresenta-se uma funo que desenha numa
janela um tringulo rectngulo issceles de cor azul, descrito pelas
coordenadas de um dos seus vrtices e pelo comprimento dos seus
catetos:
\begin{code}
fillTri :: Window -> Int -> Int -> Int -> IO()
fillTri w x y size = drawInWindow w (withColor Blue
                      (polygon [(x,y),(x+size,y),(x,y-size),(x,y)]))
\end{code}

Uma nota final: o ponto $(0,0)$ numa janela grfica corresponde ao
canto \emph{superior esquerdo} da mesma. 

\section{O Tringulo de Sierpinski}



\begin{figure}[htb]
\begin{center}
\includegraphics{sierpinski2.eps}
\end{center}
  \caption{Construo de um tringulo de Sierpinski}
  \label{fig:sierp1}
\end{figure}

\subsection{Introduo}

O tringulo de Sierpinski  uma figura com carcter fractal que se
obtm da seguinte forma: considere-se um tringulo rectngulo e
issceles $A$ cujos catetos tm comprimento $s$. A estrutura fractal 
criada desenhando-se trs tringulos no interior de $A$, todos eles
rectngulos e issceles e com catetos de comprimento $s/2$. Este passo
 depois repetido para cada um dos tringulos desenhados, e assim
sucessivamente. Os trs primeiros passos so apresentados na
Fig.~\ref{fig:sierp1}.

Note-se que um tringulo de Sierpinski  gerado repetindo-se
infinitamente o processo acima descrito; no entanto para efeitos de
visualizao num monitor (de resoluo forosamente finita) ser
necessrio escolher uma representao adequada do tringulo, parando o
processo recursivo a um determinado nvel. A figura a desenhar 
constituda por um conjunto finito de tringulos todos da mesma
dimenso (por exemplo, no quarto tringulo da figura, desenhar-se-iam
27 tringulos). Cada tringulo  geometricamente descrito pelas
coordenadas do seu vrtice inferior esquerdo e o comprimento dos seus
catetos:
\begin{code}
type Side = Int
type Tri = (Point, Side)
\end{code}

\subsection{Criao de um Tringulo de Sierpinski por um Hilomorfismo}

A estrutura recursiva de (uma representao finita de) um tringulo de
Sierpinski  captada por uma rvore ternria (cada n  um tringulo
com os respectivos trs sub-tringulos). Nas folhas dessa rvore
encontram-se os tringulos mais pequenos (todos da mesma dimenso),
que devero ser desenhados. Apenas estes contero informao de
carcter geomtrico, tendo os ns da rvore um papel exclusivamente
estrutural.

\begin{code}
data TLTree = Leaf Point Side 
           | Node TLTree TLTree TLTree
\end{code}
A informao geomtrica guardada em cada folha consiste nas
coordenadas do vrtice inferior esquerdo e no lado dos catetos do
respectivo tringulo.

A funo $\verb+sierpinski :: Point -> Side -> Int -> [Tri]+$ recebe
as coordenadas do vrtice inferior esquerdo e a dimenso do tringulo
exterior, bem como o nmero de nveis pretendido (critrio de paragem
do processo de construo do fractal), e constri uma lista de
tringulos a desenhar.  Esta funo  um hilomorfismo do tipo
\verb+TLTree+, composio das seguintes duas funes:
\begin{code}
sierpinski :: Point -> Side -> Int -> [Tri]
sierpinski p s = apresentaSierp . (geraSierp p s)

geraSierp :: Point -> Side -> Int -> TLTree
geraSierp (x,y) s 0 = Leaf (x,y) s
geraSierp (x,y) s n = Node (geraSierp (x,y) (s `div` 2) (n-1))
                           (geraSierp (x+s`div`2,y) (s `div` 2) (n-1))
                           (geraSierp (x,y-s`div`2) (s `div` 2) (n-1))

apresentaSierp :: TLTree -> [(Point, Side)]
apresentaSierp (Leaf (x,y) s) = [((x,y),s)]
apresentaSierp (Node a b c) =
             (apresentaSierp a)++(apresentaSierp b)++(apresentaSierp c)  
\end{code}


\section{Trabalho a Desenvolver}

\subsection{Sobre o Tringulo de Sierpinski}
\begin{enumerate}
\item Desenvolva a biblioteca pointfree \texttt{TLTree.hs} de forma anloga
      a outras bibliotecas que conhece (\eg\ \texttt{BTree.hs}, \etc).
\item Exprima as funes \verb+geraSierp+ e \verb+apresentaSierp+ como
      anamorfismo e catamorfismo, respectivamente, do tipo \verb+TLTree+.
\item Recorrendo s funes relevantes da biblioteca grfica, escreva
  uma funo 
\begin{verbatim}
desenha :: [Tri] -> IO() 
\end{verbatim}
que desenha uma lista de tringulos.
\item Teste o seu programa:
\begin{verbatim}
desenha (sierpinski (50,300) 256 5)
\end{verbatim}

\end{enumerate}

\subsection{Sobre o fractal Floco de Neve}
O fractal floco de neve constri-se a partir de um tringulo
equiltero de lado $s$ da seguinte forma: em cada lado do tringulo, e
centrado em relao a ele, desenha-se um tringulo (tambm ele
equiltero) de lado $s/3$, como ilustrado na segunda imagem da
Fig.~\ref{fig:snowflake1}. Na figura obtida (uma ``estrela de David'')
podemos identificar 6 tringulos de lado $s/3$, e a cada um deles
aplicar-se- de novo o passo anterior, e assim sucessivamente (mas
note-se que alguns dos tringulos desenhados depois do primeiro
sobrepem-se a tringulos j previamente desenhados pelo que podem ser
omitidos).

\begin{enumerate}
\item
Defina um tipo de dados $T$ que permita que a construo deste fractal
seja efectuada por um hilomorfismo de $T$. Defina o hilomorfismo e
teste-o recorrendo s funes da biblioteca grfica.


\begin{figure}[htb]
\begin{center}
\includegraphics{snowflake.eps}
\end{center}
  \caption{Construo de um floco de neve}
  \label{fig:snowflake1}
\end{figure}
\end{enumerate}

\section{Valorizao}
\begin{enumerate}
\item
	Modifique a sua soluo para o fractal Floco de Neve
	por forma a que tringulos de tamanhos
	diferentes sejam desenhados a cores diferentes.

\item
	Adapte a sua resposta  alnea anterior por forma a desenhar
apenas a fronteira do fractal.

\item
	Investigue outras formas fractais ---
	dispe de uma boa tutorial em
	\begin{center}
\texttt{http://uranus.ee.auth.gr/~ngramm/fractals/tutorial3.html})
	\end{center}
	--- e implemente-as tal como nos exerccios anteriores.
	Escolha exemplos relativamente simples.

\end{enumerate}

\begin{thebibliography}{1}

\bibitem{hudak00haskell}
P.~Hudak.
\newblock The haskell school of expression: Learning functional programming
  through multimedia, 2000.

\end{thebibliography}

\end{document}

